//----------------------------------------------------------------------------
//	SOURCE:	MapFlag.c
//	AUTHOR:	Ian Taylor
//	NOTES:	Source file of Map2 demo flags
//----------------------------------------------------------------------------

#include "Map.h"
#include "MapProj.h"

#define COUNTRY(a,b,d)	Flag_##a,
enum Flag
{
	Flag_None = 0,
#include "MapTable.h"
	Flag_US_Stars,
	Flag_Count
};

// Nybble (4-bit) manipulations
#define NYBBLE(hi,lo)	(((hi) << 4) + (lo))
#define LO(hilo)		((hilo) & 0xF)
#define HI(hilo)		((hilo) >> 4)

enum FlagCodeFirst
{
	FlagFirstExtend = 0xC,
	FlagFirstFlood,
	FlagFirstH3,
	FlagFirstV3,

	FlagFirstExtendNull = 0,
	FlagFirstExtendH2,
	FlagFirstExtendV2,
	FlagFirstExtendH4,
	FlagFirstExtendV4,
	FlagFirstExtendHStripe,

	FlagFirstMarker = 0xC0
};
enum FlagCode
{
	FlagExtend = 0,
	FlagSetBrush,
	FlagPolygon,
	FlagMirror,
	FlagCross,
	FlagHBlock,
	FlagVBlock,
	FlagStar,
	FlagStar5,
	FlagStarLine,
	FlagShield,

	FlagExtendTriangle = 0,
	FlagExtendRectangle,
	FlagExtendDiagonal,
	FlagExtendBlit,
	FlagExtendStar5r,
	FlagExtendStar5x,
	FlagExtendStarX,
	FlagExtendChord,
	FlagExtendSubflag
};
enum FlagColour
{
	Black = 0,
	PureBlue,
	Red,
	Magenta,
	Green,
	Cyan,
	Yellow,
	White,
	NavyBlue,
	Plum,
	Orange,
	PaleBlue,
	Blue,
	Brown,
	Grey
};

#define FLAG_BEGIN(iso)
#define FLAG_END(iso)

#define FLOODNULL()				NYBBLE(FlagFirstExtend, FlagFirstExtendNull),
#define FLOOD(c1)				NYBBLE(FlagFirstFlood,c1),
#define HBICOL(c1,c2)			NYBBLE(FlagFirstExtend, FlagFirstExtendH2), NYBBLE(c1,c2),
#define VBICOL(c1,c2)			NYBBLE(FlagFirstExtend, FlagFirstExtendV2), NYBBLE(c1,c2),
#define HTRICOL(c1,c2,c3)		NYBBLE(FlagFirstH3,c1), NYBBLE(c2,c3),
#define VTRICOL(c1,c2,c3)		NYBBLE(FlagFirstV3,c1), NYBBLE(c2,c3),
#define HQUADCOL(c1,c2,c3,c4)	NYBBLE(FlagFirstExtend, FlagFirstExtendH4), NYBBLE(c1,c2), NYBBLE(c3,c4),
#define VQUADCOL(c1,c2,c3,c4)	NYBBLE(FlagFirstExtend, FlagFirstExtendV4), NYBBLE(c1,c2), NYBBLE(c3,c4),
#define HSTRIPE(n,c1,c2)		NYBBLE(FlagFirstExtend, FlagFirstExtendHStripe), n, NYBBLE(c1,c2),

#define COLOUR(c1)				NYBBLE(FlagSetBrush,c1),
#define TRIANGLE(x0,y0,x1,y1)	NYBBLE(FlagExtend,FlagExtendTriangle), x0,y0, x1,y1,
#define RECTANGLE(x0,y0,x1,y1)	NYBBLE(FlagExtend,FlagExtendRectangle), x0,y0, x1,y1,
#define POLYGON(n)				NYBBLE(FlagPolygon,n-3)
#define MIRROR(n)				NYBBLE(FlagMirror,n-3)
#define DIAGONAL(lr,w)			NYBBLE(FlagExtend,FlagExtendDiagonal), ((lr << 6) | w-20),
#define CROSS(w,x)				NYBBLE(FlagCross,w/2-1), x,
#define HBLOCK(c1,y0,y1)		NYBBLE(FlagHBlock,c1), y0, y1,
#define VBLOCK(c1,x0,x1)		NYBBLE(FlagVBlock,c1), x0, x1,
#define STAR(n,r1,r2,x,y)		NYBBLE(FlagStar,n-4), r1,r2, x,y,
#define STAR5(r,x,y)			NYBBLE(FlagStar5,r/2-3), x,y,
#define BLIT(x0,y0,x1,y1)		NYBBLE(FlagExtend,FlagExtendBlit), x0,y0, x1,y1,
#define STAR5R(r,x,y,rot)		NYBBLE(FlagExtend, FlagExtendStar5r), r, x,y, rot,
#define STAR5X(n,x,y,r0,r1,rot)	NYBBLE(FlagExtend, FlagExtendStar5x), ((rot << 7) | n), x,y, r0,r1,
#define STARX(n,x,y,r0,r1,r2)	NYBBLE(FlagExtend, FlagExtendStarX), n, x,y, r0,r1,r2,
#define STARLINE(r,x,y,n,d)		NYBBLE(FlagStarLine,n), d, r, x,y,
#define CIRCLE(r,x,y)			NYBBLE(FlagStar,15), r,r, x,y,
#define CHORD(r,x,y,a1,a2)		NYBBLE(FlagExtend,FlagExtendChord), r, x,y, a1,a2,
#define SHIELD(x0,y0,x1,y1,out)	NYBBLE(FlagShield,out), x0,y0, x1,y1,
#define SUBFLAG(flag, aspect)	NYBBLE(FlagExtend,FlagExtendSubflag), Flag_##flag, (int)((aspect - 1) * 240),

local const BYTE raw_flag_data[] =
{
	FLAG_BEGIN(None)
		FLOODNULL()
	FLAG_END(None)
	FLAG_BEGIN(AD)
		VTRICOL(PureBlue, Yellow, Red)	//Emblem
		COLOUR(Red)
		RECTANGLE(95,75, 145,115)
		SHIELD(95,119, 145,165, 0)
		COLOUR(Yellow)
		RECTANGLE(119,0, 121,240)
	FLAG_END(AD)
	FLAG_BEGIN(AE)
		HTRICOL(Green, White, Black)
		VBLOCK(Red, 0,40)
	FLAG_END(AE)
	FLAG_BEGIN(AF)
		VTRICOL(Black, Red, Green)	//Emblem!
	FLAG_END(AF)
	FLAG_BEGIN(AG)
		FLOOD(Black)
		COLOUR(Yellow)
		STAR(16, 80,40, 120,96)
		HBLOCK(PaleBlue, 96,144)
		HBLOCK(White, 144,240)
		COLOUR(Magenta)
		TRIANGLE(0,0, 120,240)
		TRIANGLE(120,240, 240,0)
	FLAG_END(AG)
	FLAG_BEGIN(AI)
		FLOOD(NavyBlue)
		BLIT(0,0, 120,120)	//Emblem
		RECTANGLE(160,70, 200,170)
		COLOUR(Cyan)
		RECTANGLE(160,140, 200,170)
		COLOUR(NavyBlue)
		SHIELD(160,100, 200,170, 1)
		CIRCLE(25, 170,55)
		CIRCLE(25, 190,55)
		COLOUR(Orange)
		CIRCLE(25, 180,110)
		COLOUR(White)
		CIRCLE(15, 180,110)
	FLAG_END(AI)
	FLAG_BEGIN(AL)
		FLOOD(Plum)	//Emblem!
	FLAG_END(AL)
	FLAG_BEGIN(AM)
		HTRICOL(Red, PureBlue, Orange)
	FLAG_END(AM)
	FLAG_BEGIN(AO)
		HBICOL(Red, Black)	//Emblem!
	FLAG_END(AO)
	FLAG_BEGIN(AQ)
		FLOOD(White)
		BLIT(0,0, 120,120)	//Emblem!
		COLOUR(Black)
		SHIELD(159,68, 201,173, 0)
		COLOUR(White)
		SHIELD(160,70, 200,170, 0)
		COLOUR(Red)
		POLYGON(3), 170,70, 190,70, 180,140,
	FLAG_END(AQ)
	FLAG_BEGIN(AR)
		HTRICOL(Cyan, White, Cyan)
		COLOUR(Yellow)
		STAR(19, 40,20, 120,120)	//Emblem
	FLAG_END(AR)
	FLAG_BEGIN(AT)
		HTRICOL(Red, White, Red)
	FLAG_END(AT)
	FLAG_BEGIN(AU)
		FLOOD(NavyBlue)
		BLIT(0,0, 120,120)
		STAR(7, 30,10, 60,180)
		STAR(7, 21,7, 180,38)
		STAR(7, 21,7, 208,88)
		STAR(7, 21,7, 150,103)
		STAR(7, 21,7, 180,200)
		STAR5(12, 193,127)
	FLAG_END(AU)
	FLAG_BEGIN(AW)
		FLOOD(PaleBlue)
		STAR(4, 41,14, 26,52)
		COLOUR(Red)
		STAR(4, 34,11, 26,52)
		HBLOCK(Yellow, 160, 175)
		HBLOCK(Yellow, 190, 205)
	FLAG_END(AW)
	FLAG_BEGIN(AZ)
		HTRICOL(PaleBlue, Plum, Green)
		CIRCLE(34, 112,120)
		COLOUR(Plum)
		CIRCLE(28, 119,120)
		COLOUR(White)
		STAR(8, 20,10, 133,120)
	FLAG_END(AZ)
	FLAG_BEGIN(BA)
		FLOOD(PureBlue)
		STAR5(20, 40,0)
		STAR5(20, 55,30)
		STAR5(20, 70,60)
		STAR5(20, 85,90)
		STAR5(20, 100,120)
		STAR5(20, 115,150)
		STAR5(20, 130,180)
		STAR5(20, 145,210)
		STAR5(20, 160,240)
		COLOUR(Yellow)
		TRIANGLE(183,240, 64,0)
	FLAG_END(BA)
	FLAG_BEGIN(BB)
		VTRICOL(PureBlue, Yellow, PureBlue)
		COLOUR(Black)
		MIRROR(16), 120,75, 117,82, 113,86, 116,91, 116,134, 111,134, 106,88, 109,89, 101,79, 89,73, 96,88, 102,109, 107,146, 116,146, 116,166, 120,166,
	FLAG_END(BB)
	FLAG_BEGIN(BD)
		FLOOD(Green)
		COLOUR(Red)
		CIRCLE(75, 106,120)
	FLAG_END(BD)
	FLAG_BEGIN(BE)
		VTRICOL(Black, Yellow, Red)
	FLAG_END(BE)
	FLAG_BEGIN(BF)
		HBICOL(Red, Green)
		COLOUR(Yellow)
		STAR5(36, 120,123)
	FLAG_END(BF)
	FLAG_BEGIN(BG)
		HTRICOL(White, Green, Red)
	FLAG_END(BG)
	FLAG_BEGIN(BH)
		FLOOD(Red)
		POLYGON(13), 0,0, 48,0, 84,24, 48,48, 84,72, 48,96, 84,120, 48,144, 84,168, 48,192, 84,216, 48,240, 0,240,
	FLAG_END(BH)
	FLAG_BEGIN(BI)
		FLOOD(Green)
		COLOUR(Red)
		POLYGON(4), 0,0, 240,240, 240,0, 0,240,
		COLOUR(White)
		DIAGONAL(3, 20)
		CIRCLE(60, 120,120)
		COLOUR(Green)
		STAR(6, 15,9, 120,88)
		STAR(6, 15,9, 106,136)
		STAR(6, 15,9, 134,136)
		COLOUR(Red)
		STAR(6, 11,6, 120,88)
		STAR(6, 11,6, 106,136)
		STAR(6, 11,6, 134,136)
	FLAG_END(BI)
	FLAG_BEGIN(BJ)
		HBICOL(Yellow, Red)
		VBLOCK(Green, 0,80)
	FLAG_END(BJ)
	FLAG_BEGIN(BM)
		FLOOD(Red)
		BLIT(0,0, 120,120)	//Emblem
		RECTANGLE(160,70, 200,170)
		COLOUR(Red)
		SHIELD(160,150, 200,170, 1)
	FLAG_END(BM)
	FLAG_BEGIN(BN)
		FLOOD(Yellow)
		POLYGON(4), 0,0, 240,120, 240,220, 0,100,
		COLOUR(Black)
		POLYGON(4), 0,60, 240,180, 240,240, 0,120,	//Emblem
	FLAG_END(BN)
	FLAG_BEGIN(BO)
		HTRICOL(Red, Yellow, Green)
	FLAG_END(BO)
	FLAG_BEGIN(BR)
		FLOOD(Green)
		COLOUR(Yellow)
		POLYGON(4), 120,20, 220,120, 120,220, 20,120,
		COLOUR(PureBlue)
		CIRCLE(68, 120,120)	//Emblem
	FLAG_END(BR)
	FLAG_BEGIN(BS)
		HTRICOL(Cyan, Yellow, Cyan)
		COLOUR(Black)
		POLYGON(3), 0,0, 107,120, 0,240,
	FLAG_END(BS)
	FLAG_BEGIN(BT)
		FLOOD(Yellow)
		COLOUR(Orange)
		TRIANGLE(0,240, 240,0)	//Emblem
	FLAG_END(BT)
	FLAG_BEGIN(BW)
		FLOOD(Cyan)
		HBLOCK(White, 92, 148)
		HBLOCK(Black, 100, 140)
	FLAG_END(BW)
	FLAG_BEGIN(BY)
		HTRICOL(Red, Red, Green)
		VBLOCK(White, 0,25)	//Emblem
	FLAG_END(BY)
	FLAG_BEGIN(BZ)
		FLOOD(Red)
		HBLOCK(PureBlue, 25, 215)
		CIRCLE(84, 120,120)	//Emblem
	FLAG_END(BZ)
	FLAG_BEGIN(CA)
		FLOOD(Red)
		VBLOCK(White, 60, 180)
		COLOUR(Red)
		MIRROR(14), 120,14, 110,50, 101,39, 106,102, 91,69, 88,85, 72,79, 77,116, 70,122, 97,165, 92,182, 118,176, 117,228, 120,228,
	FLAG_END(CA)
	FLAG_BEGIN(CD)
		FLOOD(Blue)
		COLOUR(Yellow)
		STAR5R(60, 120,120, 0)
		STAR5(18, 20,25)
		STAR5(18, 20,63)
		STAR5(18, 20,101)
		STAR5(18, 20,139)
		STAR5(18, 20,177)
		STAR5(18, 20,215)
	FLAG_END(CD)
	FLAG_BEGIN(CF)
		HQUADCOL(Yellow, PureBlue, White, Green)
		VBLOCK(Red, 100, 140)
		COLOUR(Yellow)
		STAR5(26, 30,30)
	FLAG_END(CF)
	FLAG_BEGIN(CG)
		FLOOD(Yellow)
		COLOUR(Green)
		TRIANGLE(160,0, 0,240)
		COLOUR(Red)
		TRIANGLE(80,240, 240,0)
	FLAG_END(CG)
	FLAG_BEGIN(CH)
		FLOOD(Red)
		RECTANGLE(96,48, 144,192)
		RECTANGLE(48,96, 192,144)
	FLAG_END(CH)
	FLAG_BEGIN(CI)
		VTRICOL(Orange, White, Green)
	FLAG_END(CI)
	FLAG_BEGIN(CK)
		FLOOD(NavyBlue)
		BLIT(0,0, 120,120)
		STAR5X(15, 180,120, 91,11, 1)
	FLAG_END(CK)
	FLAG_BEGIN(CL)
		HBICOL(White, Red)
		COLOUR(PureBlue)
		RECTANGLE(0,0, 80,120)
		COLOUR(White)
		STAR5(36, 40,60)
	FLAG_END(CL)
	FLAG_BEGIN(CM)
		VTRICOL(Green, Red, Yellow)
		COLOUR(Yellow)
		STAR5(36, 120,123)
	FLAG_END(CM)
	FLAG_BEGIN(CN)
		FLOOD(Red)
		COLOUR(Yellow)
		STAR5(36, 40,60)
		STAR5R(12, 80,20, 18)
		STAR5R(12, 96,45, 45)
		STAR5R(12, 96,83, 63)
		STAR5R(12, 80,108, 18)
	FLAG_END(CN)
	FLAG_BEGIN(CO)
		HQUADCOL(Yellow, Yellow, PureBlue, Red)
	FLAG_END(CO)
	FLAG_BEGIN(CR)
		FLOOD(Blue)
		HBLOCK(White, 40, 200)
		HBLOCK(Red, 80, 160)
	FLAG_END(CR)
	FLAG_BEGIN(CS)
		HTRICOL(PureBlue, White, Red)
	FLAG_END(CS)
	FLAG_BEGIN(CU)
		HSTRIPE(5, Blue,White)
		COLOUR(Red)
		POLYGON(3), 0,0, 107,120, 0,240,
		COLOUR(White)
		STAR5(36, 35,123)
	FLAG_END(CU)
	FLAG_BEGIN(CV)
		HQUADCOL(Blue, Blue, White, Blue)
		HBLOCK(Red, 140, 160)
		COLOUR(Yellow)
		STAR5X(10, 67,150, 54,12, 0)
	FLAG_END(CV)
	FLAG_BEGIN(CY)
		FLOOD(White)
		COLOUR(Yellow)	//Emblem
		POLYGON(18), 177,71, 152,86, 124,90, 107,78, 101,82, 102,93, 89,88, 81,94, 76,88, 70,100, 74,121, 88,132, 98,139, 104,131, 126,130, 132,119, 147,121, 145,105,
	FLAG_END(CY)
	FLAG_BEGIN(CZ)
		HBICOL(White, Red)
		COLOUR(Blue)
		POLYGON(3), 0,0, 120,120, 0,240,
	FLAG_END(CZ)
	FLAG_BEGIN(DE)
		HTRICOL(Black, Red, Orange)
	FLAG_END(DE)
	FLAG_BEGIN(DJ)
		HBICOL(PureBlue, Green)
		POLYGON(3), 0,0, 96,120, 0,240,
		COLOUR(Plum)
		STAR5(36, 32,123)
	FLAG_END(DJ)
	FLAG_BEGIN(DK)
		FLOOD(Red)
		CROSS(12, 90)
	FLAG_END(DK)
	FLAG_BEGIN(DM)
		FLOOD(Green)
		CROSS(18, 120)
		VBLOCK(Yellow, 102, 120)
		HBLOCK(Yellow, 84, 120)
		COLOUR(Black)
		CROSS(6, 120)
		COLOUR(Red)
		CIRCLE(76, 120,120)
		COLOUR(Yellow)
		STAR5X(10, 120,120, 58,14, 1)
		COLOUR(Green)
		STAR5X(10, 120,120, 58,10, 1)
	FLAG_END(DM)	//Emblem!
	FLAG_BEGIN(DO)
		FLOOD(Red)
		COLOUR(PureBlue)
		POLYGON(6), 0,0, 120,0, 120,240, 240,240, 240,120, 0,120,
		COLOUR(White)
		CROSS(12, 120)
	FLAG_END(DO)
	FLAG_BEGIN(DZ)
		VBICOL(Green, White)
		COLOUR(Red)
		CIRCLE(70, 120,120)
		COLOUR(White)
		CIRCLE(52, 133,120)
		COLOUR(Red)
		RECTANGLE(100,70, 120,170)
		STAR5R(36, 135,120, 18)
		COLOUR(Green)
		CHORD(52, 133,120, 140, 220)
	FLAG_END(DZ)
	FLAG_BEGIN(EC)
		HQUADCOL(Yellow, Yellow, Blue, Red)
	FLAG_END(EC)
	FLAG_BEGIN(EE)
		HTRICOL(PureBlue, Black, White)
	FLAG_END(EE)
	FLAG_BEGIN(EG)
		HTRICOL(Red, White, Black)
		COLOUR(Orange)	//Emblem
		POLYGON(16), 119,81, 113,84, 117,86, 115,99, 109,94, 107,143, 112,131, 109,152, 116,157, 124,157, 131,152, 128,131, 133,143, 131,94, 125,99, 123,84,
	FLAG_END(EG)
	FLAG_BEGIN(EH)
		HTRICOL(Black, White, Green)
		COLOUR(Plum)
		POLYGON(3), 0,0, 60,120, 0,240,
		CIRCLE(38, 120,120)
		COLOUR(White)
		CIRCLE(30, 126,120)
		COLOUR(Plum)
		STAR5R(22, 138,120, 54)
	FLAG_END(EH)
	FLAG_BEGIN(ER)
		HBICOL(Green, PureBlue)
		COLOUR(Red)
		POLYGON(3), 0,0, 240,120, 0,240,
	FLAG_END(ER)	//Emblem!
	FLAG_BEGIN(ES)
		HQUADCOL(Red, Yellow, Yellow, Red)	//Emblem
		COLOUR(Red)
		RECTANGLE(60,90, 90,150)
		COLOUR(White)
		RECTANGLE(75,90, 90,120)
		RECTANGLE(60,120, 75,150)
		COLOUR(Orange)
		RECTANGLE(60,120, 75,150)
		COLOUR(Yellow)
		SHIELD(60,140, 90,150, 1)
	FLAG_END(ES)
	FLAG_BEGIN(ET)
		HTRICOL(Green, Yellow, Red)
		COLOUR(Cyan)
		CIRCLE(60, 120,120)
		COLOUR(Yellow)
		STAR(5, 5,56, 120,120)
		STAR5R(52, 120,120, 0)
		COLOUR(Cyan)
		STAR5(36, 120,120)
	FLAG_END(ET)
	FLAG_BEGIN(FI)
		FLOOD(White)
		COLOUR(PaleBlue)
		CROSS(17, 80)
	FLAG_END(FI)
	FLAG_BEGIN(FJ)
		FLOOD(Cyan)
		BLIT(0,0, 120,120)	//Emblem
		RECTANGLE(160,70, 200,170)
		COLOUR(Red)
		RECTANGLE(160,70, 200,100)
		RECTANGLE(160,120, 200,135)
		RECTANGLE(176,100, 184,170)
		COLOUR(Cyan)
		SHIELD(160,120, 200,170, 1)
	FLAG_END(FJ)
	FLAG_BEGIN(FM)
		FLOOD(PaleBlue)
		STAR5X(4, 120,120, 68,22, 1)
	FLAG_END(FM)
	FLAG_BEGIN(FR)
		VTRICOL(Blue, White, Red)
	FLAG_END(FR)
	FLAG_BEGIN(GA)
		HTRICOL(Green, Yellow, PureBlue)
	FLAG_END(GA)
	FLAG_BEGIN(GB)
		FLOOD(NavyBlue)
		DIAGONAL(3, 27)
		COLOUR(Red)
		POLYGON(12), 0,0, 111,111, 222,0, 240,0, 129,111, 240,222, 240,240, 129,129, 18,240, 0,240, 111,129, 0,18,
		COLOUR(White)
		CROSS(20, 120)
		COLOUR(Red)
		CROSS(12, 120)
	FLAG_END(GB)
	FLAG_BEGIN(GD)
		FLOOD(Red)
		COLOUR(Yellow)
		RECTANGLE(24,36, 216,204)
		COLOUR(Green)
		POLYGON(4), 24,36, 216,204, 216,36, 24,204,
		COLOUR(Red)
		CIRCLE(22, 120,120)
		COLOUR(Yellow)
		STAR5(20, 70,19)
		STAR5(20, 120,19)
		STAR5(20, 170,19)
		STAR5(20, 120,120)
		STAR5(20, 70,223)
		STAR5(20, 120,223)
		STAR5(20, 170,223)
	FLAG_END(GD)	//Emblem
	FLAG_BEGIN(GE)
		FLOOD(Plum)
		RECTANGLE(0,0, 100,92)
		COLOUR(Black)
		RECTANGLE(0,0, 100,46)
	FLAG_END(GE)
	FLAG_BEGIN(GF)
		VTRICOL(Blue, White, Red)
	FLAG_END(GF)
	FLAG_BEGIN(GH)
		HTRICOL(Red, Yellow, Green)
		COLOUR(Black)
		STAR5R(43, 120,123, 0)
	FLAG_END(GH)
	FLAG_BEGIN(GI)
		HTRICOL(White, White, Red)	//Emblem
		COLOUR(Red)
		MIRROR(13), 120,33, 111,33, 109,81, 107,45, 92,45, 90,81, 87,81, 87,97, 90,102, 86,138, 84,138, 84,148, 120,148,
		COLOUR(Yellow)
		POLYGON(16), 118,138, 118,211, 112,211, 112,216, 115,216, 115,213, 118,213, 118,224, 115,224, 115,221, 112,221, 112,226, 118,226, 118,232, 122,232, 122,138,
		CIRCLE(10, 120,170)
		COLOUR(Red)
		CIRCLE(4, 120,170)
	FLAG_END(GI)
	FLAG_BEGIN(GL)
		HBICOL(White, Plum)
		CHORD(75, 70,120, 60,180)
		COLOUR(Plum)
		CHORD(75, 70,120, 180,60)
	FLAG_END(GL)
	FLAG_BEGIN(GM)
		HTRICOL(Red, Blue, Green)
		HBLOCK(White, 74, 86)
		HBLOCK(White, 154, 166)
	FLAG_END(GM)
	FLAG_BEGIN(GN)
		VTRICOL(Red, Yellow, Green)
	FLAG_END(GN)
	FLAG_BEGIN(GP)
		HTRICOL(Blue, Black, Black)
		COLOUR(Yellow)
		STAR(16, 75,35, 120,160)
		STAR(16, 35,75, 120,160)
	FLAG_END(GP)	//Emblem
	FLAG_BEGIN(GQ)
		HTRICOL(Green, White, Red)
		COLOUR(PaleBlue)
		POLYGON(3), 0,0, 44,120, 0,240,
	FLAG_END(GQ)	//Emblem
	FLAG_BEGIN(GR)
		HSTRIPE(9, Blue, White)
		COLOUR(Blue)
		RECTANGLE(0,0, 90,133)
		COLOUR(White)
		RECTANGLE(0,53, 90,80)
		RECTANGLE(36,0, 54,133)
	FLAG_END(GR)
	FLAG_BEGIN(GT)
		VTRICOL(PaleBlue, White, PaleBlue)
	FLAG_END(GT)
	FLAG_BEGIN(GU)
		FLOOD(Red)
		COLOUR(Blue)
		RECTANGLE(2,4, 238,236)
		COLOUR(Red)
		CHORD(120, 79,120, 29,91)
		CHORD(120, 161,120, 149,211)
		COLOUR(PaleBlue)
		CHORD(114, 79,120, 31,89)
		CHORD(114, 161,120, 151,209)
	FLAG_END(GU)	//Emblem
	FLAG_BEGIN(GW)
		HBICOL(Yellow, Green)
		VBLOCK(Red, 0,80)
		COLOUR(Black)
		STAR5(36, 40,123)
	FLAG_END(GW)
	FLAG_BEGIN(GY)
		FLOOD(Green)
		POLYGON(3), 0,0, 240,120, 0,240,
		COLOUR(Yellow)
		POLYGON(3), 0,10, 220,120, 0,230,
		COLOUR(Black)
		POLYGON(3), 0,0, 120,120, 0,240,
		COLOUR(Red)
		POLYGON(3), 0,10, 110,120, 0,230,
	FLAG_END(GY)
	FLAG_BEGIN(HK)
		FLOOD(Red)
		POLYGON(16), 120,120, 107,89, 112,51, 132,35, 132,78, 119,96, 121,160, 106,185, 85,175, 98,140, 114,141, 131,86, 150,84, 164,116, 144,129, 131,110,
		POLYGON(11), 120,120, 101,135, 85,114, 83,71, 104,85, 108,115, 139,130, 147,166, 135,203, 123,168, 128,138,
		COLOUR(Red)
		STAR5X(5, 120,120, 50,8, 1)
	FLAG_END(HK)
	FLAG_BEGIN(HN)
		HTRICOL(Blue, White, Blue)
		COLOUR(Blue)
		STAR5(16, 80,100)
		STAR5(16, 80,140)
		STAR5(16, 120,120)
		STAR5(16, 160,100)
		STAR5(16, 160,140)
	FLAG_END(HN)
	FLAG_BEGIN(HR)
		HTRICOL(Red, White, PureBlue)
		RECTANGLE( 95,160, 145,180)
		COLOUR(Red)
		RECTANGLE(105, 80, 115,100)
		RECTANGLE(125, 80, 135,100)
		RECTANGLE( 95,100, 105,120)
		RECTANGLE(115,100, 125,120)
		RECTANGLE(135,100, 145,120)
		RECTANGLE(105,120, 115,140)
		RECTANGLE(125,120, 135,140)
		RECTANGLE( 95,140, 105,160)
		RECTANGLE(115,140, 125,160)
		RECTANGLE(135,140, 145,160)
		RECTANGLE(105,160, 115,180)
		RECTANGLE(125,160, 135,180)
		COLOUR(PureBlue)
		SHIELD( 95,160, 145,180, 1)
	FLAG_END(HR)	//Emblem
	FLAG_BEGIN(HT)
		HBICOL(Blue, Red)
	FLAG_END(HT)
	FLAG_BEGIN(HU)
		HTRICOL(Red, White, Green)
	FLAG_END(HU)
	FLAG_BEGIN(ID)
		HBICOL(Red, White)
	FLAG_END(ID)
	FLAG_BEGIN(IE)
		VTRICOL(Green, White, Orange)
	FLAG_END(IE)
	FLAG_BEGIN(IL)
		FLOOD(White)
		HBLOCK(PureBlue, 20, 60)
		HBLOCK(PureBlue, 180, 220)
		COLOUR(PureBlue)
		STAR(6, 52,30, 120,120)
		COLOUR(White)
		STAR(6, 30,17, 120,120)
		COLOUR(PureBlue)
		STAR(6, 26,30, 120,120)
		COLOUR(White)
		STAR(6, 15,17, 120,120)
	FLAG_END(IL)
	FLAG_BEGIN(IN)
		HTRICOL(Yellow, White, Green)
		COLOUR(Blue)
		CIRCLE(40, 120,120)
		COLOUR(White)
		STAR(19, 40,25, 120,120)
		COLOUR(Blue)
		STAR(19, 10,25, 120,120)
	FLAG_END(IN)
	FLAG_BEGIN(IQ)
		HTRICOL(Red, White, Black)
		COLOUR(Green)
		STAR5(36, 60,123)
		STAR5(36, 120,123)
		STAR5(36, 180,123)
	FLAG_END(IQ)
	FLAG_BEGIN(IR)
		HTRICOL(Green, White, Red)
		COLOUR(Red)
		CIRCLE(30, 115,120)
		CIRCLE(30, 125,120)
		COLOUR(White)
		CIRCLE(35, 120,124)
		COLOUR(Red)
		CIRCLE(29, 120,123)
		COLOUR(White)
		CIRCLE(25, 120,118)
		COLOUR(Red)
		RECTANGLE(118,90, 122,150)
	FLAG_END(IR)
	FLAG_BEGIN(IS)
		FLOOD(PureBlue)
		CROSS(18, 80)
		COLOUR(Red)
		CROSS(10, 80)
	FLAG_END(IS)
	FLAG_BEGIN(IT)
		VTRICOL(Green, White, Red)
	FLAG_END(IT)
	FLAG_BEGIN(JM)
		FLOOD(Green)
		COLOUR(Black)
		POLYGON(4), 0,0, 240,240, 240,0, 0,240,
		COLOUR(Yellow)
		DIAGONAL(3, 20)
	FLAG_END(JM)
	FLAG_BEGIN(JO)
		HTRICOL(Black, White, Green)
		COLOUR(Red)
		POLYGON(3), 0,0, 120,120, 0,240,
		COLOUR(White)
		STAR(7, 20,8, 40,120)
	FLAG_END(JO)
	FLAG_BEGIN(JP)
		FLOOD(White)
		COLOUR(Red)
		CIRCLE(70, 120,120)
	FLAG_END(JP)
	FLAG_BEGIN(KE)
		HTRICOL(Black, Red, Green)
		HBLOCK(White, 77,83)
		HBLOCK(White, 157,163)
		MIRROR(10), 120,119, 104,80, 104,70, 94,56, 98,76, 102,79, 119,120, 96,179, 97,181, 120,121,
		COLOUR(Red)
		CHORD(83, 149,120, 150,210)
		CHORD(83,  91,120,  30, 90)
		COLOUR(Black)
		CHORD(83, 149,120, 165,195)
		CHORD(84, 168,120, 165,195)
		CHORD(83,  91,120,  45, 75)
		CHORD(84,  72,120,  45, 75)
		COLOUR(White)
		CHORD(48, 141, 90, 165,195)
		CHORD(48,  99, 90,  45, 75)
		CHORD(48, 141,150, 165,195)
		CHORD(48,  99,150,  45, 75)
		CIRCLE(5, 120,120)
	FLAG_END(KE)
	FLAG_BEGIN(KG)
		FLOOD(Red)
		COLOUR(Yellow)
		STAR(19, 90,40, 120,120)
		STAR(19, 40,90, 120,120)
		COLOUR(Red)
		CIRCLE(50, 120,120)
		COLOUR(Yellow)
		CIRCLE(40, 120,120)
	FLAG_END(KG)	//Emblem?
	FLAG_BEGIN(KH)
		HQUADCOL(Blue, Red, Red, Blue)
		MIRROR(10), 120,65, 112,83, 107,111, 95,111, 85,84, 75,111, 76,142, 62,166, 62,173, 120,173,
	FLAG_END(KH)	//Emblem?
	FLAG_BEGIN(KI)
		FLOOD(Red)
		COLOUR(Yellow)
		STAR(19, 80,40, 120,128)
		COLOUR(White)
		MIRROR(17), 120,134, 111,131, 102,125, 93,122, 84,125, 75,131, 66,134, 57,131, 48,124, 39,122, 30,126, 21,132, 12,134, 6,132, 0,128, 0,240, 120,240,
		COLOUR(NavyBlue)
		MIRROR(17), 120,154, 111,151, 102,145, 93,142, 84,145, 75,151, 66,154, 57,151, 48,144, 39,142, 30,146, 21,152, 12,154, 6,152, 0,148, 0,240, 120,240,
		COLOUR(White)
		MIRROR(17), 120,174, 111,171, 102,165, 93,162, 84,165, 75,171, 66,174, 57,171, 48,164, 39,162, 30,166, 21,172, 12,174, 6,172, 0,168, 0,240, 120,240,
		COLOUR(NavyBlue)
		MIRROR(17), 120,194, 111,191, 102,185, 93,182, 84,185, 75,191, 66,194, 57,191, 48,184, 39,182, 30,186, 21,192, 12,194, 6,192, 0,188, 0,240, 120,240,
		COLOUR(White)
		MIRROR(17), 120,214, 111,211, 102,205, 93,202, 84,205, 75,211, 66,214, 57,211, 48,204, 39,202, 30,206, 21,212, 12,214, 6,212, 0,208, 0,240, 120,240,
		COLOUR(NavyBlue)
		MIRROR(17), 120,234, 111,231, 102,225, 93,222, 84,225, 75,231, 66,234, 57,231, 48,224, 39,222, 30,226, 21,232, 12,234, 6,232, 0,228, 0,240, 120,240,
	FLAG_END(KI)	//Emblem?
	FLAG_BEGIN(KM)
		HQUADCOL(Yellow, White, Red, Blue)
		COLOUR(Green)
		POLYGON(3), 0,0, 120,120, 0,240,
		COLOUR(White)
		CIRCLE(60, 40,120)
		COLOUR(Green)
		CIRCLE(60, 50,120)
		COLOUR(White)
		STAR5(15, 46,76)
		STAR5(15, 46,106)
		STAR5(15, 46,136)
		STAR5(15, 46,166)
	FLAG_END(KM)
	FLAG_BEGIN(KN)
		FLOOD(Red)
		COLOUR(Green)
		TRIANGLE(240,0, 0,240)
		COLOUR(Yellow)
		DIAGONAL(2, 83)
		COLOUR(Black)
		DIAGONAL(2, 60)
		COLOUR(White)
		STAR5R(40, 80,160, 36)
		STAR5R(40, 160,80, 36)
	FLAG_END(KN)
	FLAG_BEGIN(KP)
		FLOOD(PureBlue)
		HBLOCK(White, 40,200)
		HBLOCK(Red, 46,194)
		CIRCLE(55, 79,120)
		COLOUR(Red)
		STAR5R(55, 79,120, 0)
	FLAG_END(KP)
	FLAG_BEGIN(KR)
		FLOOD(White)
		COLOUR(Red)
		CHORD(75, 120,120, 195,75)
		COLOUR(Blue)
		CHORD(75, 120,120, 75,195)
		CIRCLE(37, 137,135)
		COLOUR(Red)
		CIRCLE(37, 103,105)
		COLOUR(Black)
		POLYGON(4), 55,75, 75,35, 70,30, 50,70,
		POLYGON(4), 48,68, 68,28, 63,23, 43,63,
		POLYGON(4), 41,61, 61,21, 56,16, 36,56,
		POLYGON(4), 185,75, 176,57, 181,52, 190,70,
		POLYGON(4), 174,53, 165,35, 170,30, 179,48,
		POLYGON(4), 192,68, 172,28, 177,23, 197,63,
		POLYGON(4), 199,61, 190,43, 195,38, 204,56,
		POLYGON(4), 188,39, 179,21, 184,16, 193,34,
		POLYGON(4), 55,165, 75,205, 70,210, 50,170,
		POLYGON(4), 48,172, 57,190, 52,195, 43,177,
		POLYGON(4), 59,194, 68,212, 63,217, 54,199,
		POLYGON(4), 41,179, 61,219, 56,224, 36,184,
		POLYGON(4), 174,187, 165,205, 170,210, 179,192,
		POLYGON(4), 181,194, 172,212, 177,217, 186,199,
		POLYGON(4), 188,201, 179,219, 184,224, 193,206,
		POLYGON(4), 185,165, 176,183, 181,188, 190,170,
		POLYGON(4), 192,172, 183,190, 188,195, 197,177,
		POLYGON(4), 199,179, 190,197, 195,202, 204,184,
	FLAG_END(KR)
	FLAG_BEGIN(KW)
		HTRICOL(Green, White, Red)
		COLOUR(Black)
		POLYGON(4), 0,0, 60,80, 60,160, 0,240,
	FLAG_END(KW)
	FLAG_BEGIN(KY)
		FLOOD(NavyBlue)
		BLIT(0,0, 120,120)	//Emblem
		RECTANGLE(160,70, 200,170)
		COLOUR(Red)
		RECTANGLE(160,70, 200,100)
		COLOUR(Green)
		STAR5(15, 170,120)
		STAR5(15, 190,120)
		STAR5(15, 180,145)
		COLOUR(NavyBlue)
		SHIELD(160,120, 200,170, 1)
	FLAG_END(KY)
	FLAG_BEGIN(KZ)
		FLOOD(Cyan)
		VBLOCK(Yellow, 10,24)
		COLOUR(Yellow)
		STAR(16, 80,40, 120,120)
		STAR(16, 40,80, 120,120)
		COLOUR(Cyan)
		CIRCLE(60, 120,120)
		COLOUR(Yellow)
		CIRCLE(55, 120,120)
	FLAG_END(KZ)
	FLAG_BEGIN(LA)
		HQUADCOL(Red, Blue, Blue, Red)
		CIRCLE(45, 120,120)
	FLAG_END(LA)
	FLAG_BEGIN(LB)
		HQUADCOL(Red, White, White, Red)
		COLOUR(Green)
		MIRROR(14), 120,65, 102,87, 111,85, 90,106, 109,101, 84,127, 104,122, 79,146, 100,141, 82,155, 108,150, 106,164, 93,173, 120,167,
	FLAG_END(LB)
	FLAG_BEGIN(LC)
		FLOOD(Cyan)
		POLYGON(3), 80,220, 120,20, 160,220,
		COLOUR(Black)
		POLYGON(3), 85,220, 120,45, 155,220,
		COLOUR(Yellow)
		POLYGON(3), 80,220, 120,130, 160,220,
	FLAG_END(LC)
	FLAG_BEGIN(LI)
		HBICOL(Blue, Red)
		COLOUR(Yellow)
		MIRROR(14), 60,25, 59,25, 59,31, 55,31, 55,35, 59,35, 59,41, 55,45, 46,43, 39,45, 35,52, 35,74, 40,90, 60,90,
	FLAG_END(LI)	//Emblem
	FLAG_BEGIN(LK)
		FLOOD(Yellow)
		COLOUR(Green)
		RECTANGLE(10,20, 45,220)
		COLOUR(Orange)
		RECTANGLE(45,20, 80,220)
		COLOUR(Brown)
		RECTANGLE(90,20, 230,220)
	FLAG_END(LK)	//Emblem
	FLAG_BEGIN(LR)
		HSTRIPE(11, Red,White)
		COLOUR(NavyBlue)
		RECTANGLE(0,0, 54,109)
		COLOUR(White)
		STAR5(36, 27,54)
	FLAG_END(LR)
	FLAG_BEGIN(LS)
		FLOOD(White)
		COLOUR(Blue)
		TRIANGLE(0,240, 240,0)
		COLOUR(Green)
		TRIANGLE(60,240, 240,60)
	FLAG_END(LS)	//Emblem
	FLAG_BEGIN(LT)
		HTRICOL(Yellow, Green, Red)
	FLAG_END(LT)
	FLAG_BEGIN(LU)
		HTRICOL(Red, White, PaleBlue)
	FLAG_END(LU)
	FLAG_BEGIN(LV)
		FLOOD(Plum)
		HBLOCK(White, 90,150)
	FLAG_END(LV)
	FLAG_BEGIN(LY)
		FLOOD(Green)
	FLAG_END(LY)
	FLAG_BEGIN(MA)
		FLOOD(Red)
		COLOUR(Green)
		STAR(5, 60,23, 120,120)
		COLOUR(Red)
		STAR(5, 40,15, 120,120)
		COLOUR(Green)
		STAR(5, 19,23, 120,120)
		COLOUR(Red)
		STAR(5, 12,15, 120,120)
	FLAG_END(MA)
	FLAG_BEGIN(MC)
		HBICOL(Red, White)
	FLAG_END(MC)
	FLAG_BEGIN(MD)
		VTRICOL(PureBlue, Yellow, Red)	//Emblem
		COLOUR(Red)
		RECTANGLE(100,70, 140,120)
		COLOUR(PureBlue)
		RECTANGLE(100,120, 140,170)
		COLOUR(Yellow)
		SHIELD(100,150, 140,170, 1)
	FLAG_END(MD)
	FLAG_BEGIN(MG)
		HBICOL(Red, Green)
		VBLOCK(White, 0,80)
	FLAG_END(MG)
	FLAG_BEGIN(MH)
		FLOOD(NavyBlue)
		STAR(12, 55,20, 50,80)
		STAR(12, 20,55, 50,80)
		STAR(4, 75,10, 50,80)
		POLYGON(3), 0,240, 240,0, 240,106,
		COLOUR(Orange)
		POLYGON(3), 0,240, 240,0, 240,53,
	FLAG_END(MH)
	FLAG_BEGIN(MK)
		FLOOD(Red)
		COLOUR(Yellow)
		MIRROR(14), 120,0, 132,0, 120,120, 196,0, 240,0, 120,120, 240,94, 240,146, 120,120, 240,240, 196,240, 120,120, 132,240, 120,240,
		COLOUR(Red)
		CIRCLE(40, 120,120)
		COLOUR(Yellow)
		CIRCLE(32, 120,120)
	FLAG_END(MK)
	FLAG_BEGIN(ML)
		VTRICOL(Green, Yellow, Red)
	FLAG_END(ML)
	FLAG_BEGIN(MM)
		FLOOD(Red)
		COLOUR(PureBlue)
		RECTANGLE(0,0, 90,120)
		COLOUR(White)
		STAR5X(14, 45,60, 40,6, 0)
		STAR(14, 25,20, 45,60)
	FLAG_END(MM)
	FLAG_BEGIN(MN)
		VTRICOL(Red, PureBlue, Red)
		COLOUR(Yellow)
		CIRCLE(30, 40,60)
		CIRCLE(20, 40,156)
		COLOUR(Red)
		CIRCLE(30, 40,52)
		CHORD(12, 40,145, 0,120)
		CHORD(12, 40,167, 120,0)
		COLOUR(Yellow)
		CIRCLE(20, 40,55)
		CIRCLE(9, 40,145)
		CIRCLE(9, 40,167)
		MIRROR(5), 40,10, 37,16, 35,13, 35,23, 40,32,
		POLYGON(3), 30,96, 50,96, 40,108,
		POLYGON(3), 30,206, 50,206, 40,218,
		RECTANGLE(30,117, 50,130)
		RECTANGLE(30,185, 50,198)
		RECTANGLE(16,96, 22,218)
		RECTANGLE(58,96, 64,218)
		COLOUR(Red)
		CIRCLE(2, 40,145)
		CIRCLE(2, 40,167)
	FLAG_END(MN)
	FLAG_BEGIN(MP)
		FLOOD(PaleBlue)
		COLOUR(Grey)
		CHORD(30, 120,70, 60,180)
		POLYGON(4), 127,70, 135,175, 105,175, 113,70,
		COLOUR(White)
		STAR5R(55, 120,120, 0)
		STARX(30, 120,120, 80,10,10)
	FLAG_END(MP)	//Emblem
	FLAG_BEGIN(MQ)
		FLOOD(Blue)
		CROSS(10, 120)
	FLAG_END(MQ)	//Emblem
	FLAG_BEGIN(MR)
		FLOOD(Green)
		COLOUR(Yellow)
		CIRCLE(80, 120,100)
		COLOUR(Green)
		CIRCLE(80, 120,80)
		COLOUR(Yellow)
		STAR5(36, 120,80)
	FLAG_END(MR)
	FLAG_BEGIN(MT)
		VBICOL(White, Red)
		COLOUR(Red)
		RECTANGLE(5,30, 45,50)
		RECTANGLE(20,5, 30,75)
		CIRCLE(23, 25,40)
		COLOUR(Grey)
		RECTANGLE(7,34, 43,46)
		RECTANGLE(22,9, 28,71)
		CIRCLE(20, 25,40)
	FLAG_END(MT)	//Emblem
	FLAG_BEGIN(MU)
		HQUADCOL(Red, PureBlue, Yellow, Green)
	FLAG_END(MU)
	FLAG_BEGIN(MV)
		FLOOD(Red)
		COLOUR(Green)
		RECTANGLE(40,60, 200,180)
		COLOUR(White)
		CIRCLE(40, 130,120)
		COLOUR(Green)
		CIRCLE(40, 140,120)
	FLAG_END(MV)
	FLAG_BEGIN(MW)
		HTRICOL(Black, Red, Green)
		COLOUR(Red)
		CIRCLE(70, 120,80)
		COLOUR(Black)
		STAR(19, 75,35, 120,80)
		STAR(19, 35,75, 120,80)
		COLOUR(Red)
		CIRCLE(40, 120,80)
		HBLOCK(Black, 73,80)
		HBLOCK(Red, 80,160)
	FLAG_END(MW)
	FLAG_BEGIN(MX)
		VTRICOL(Green, White, Red)	//Emblem
	FLAG_END(MX)
	FLAG_BEGIN(MY)
		HSTRIPE(14, Red,White)
		COLOUR(PureBlue)
		RECTANGLE(0,0, 120,137)
		COLOUR(Yellow)
		CIRCLE(55, 50,69)
		COLOUR(PureBlue)
		CIRCLE(48, 57,69)
		COLOUR(Yellow)
		STAR(14, 37,17, 77,69)
	FLAG_END(MY)
	FLAG_BEGIN(MZ)
		HTRICOL(Green, Black, Yellow)
		HBLOCK(White, 80, 88)
		HBLOCK(White, 152, 160)
		COLOUR(Red)
		POLYGON(3), 0,0, 96,120, 0,240,
		COLOUR(Yellow)
		STAR5R(45, 32,120, 0)
	FLAG_END(MZ)
	FLAG_BEGIN(NA)
		FLOOD(White)
		COLOUR(Red)
		DIAGONAL(2, 50)
		COLOUR(Blue)
		TRIANGLE(180,0, 0,180)
		COLOUR(Green)
		TRIANGLE(60,240, 240,60)
		COLOUR(Yellow)
		STAR(12, 45,25, 45,60)
		COLOUR(Blue)
		CIRCLE(25, 45,60)
		COLOUR(Yellow)
		CIRCLE(20, 45,60)
	FLAG_END(NA)
	FLAG_BEGIN(NC)
		VTRICOL(Blue, White, Red)
	FLAG_END(NC)
	FLAG_BEGIN(NE)
		HTRICOL(Orange, White, Green)
		COLOUR(Orange)
		CIRCLE(30, 120,120)
	FLAG_END(NE)
	FLAG_BEGIN(NG)
		VTRICOL(Green, White, Green)
	FLAG_END(NG)
	FLAG_BEGIN(NI)
		HTRICOL(PaleBlue, White, PaleBlue)
	FLAG_END(NI)
	FLAG_BEGIN(NL)
		HTRICOL(Red, White, Blue)
	FLAG_END(NL)
	FLAG_BEGIN(NO)
		FLOOD(Red)
		CROSS(18, 80)
		COLOUR(Blue)
		CROSS(10, 80)
	FLAG_END(NO)
	FLAG_BEGIN(NP)
		FLOODNULL()
		COLOUR(Blue)
		TRIANGLE(0,0, 240,120)
		TRIANGLE(0,60, 240,240)
		COLOUR(Plum)
		TRIANGLE(4,10, 208,112)
		TRIANGLE(4,71, 216,230)
		COLOUR(White)
		STAR(16, 32,20, 40,175)
		CHORD(35, 40,65, 60,180)
		COLOUR(Plum)
		CHORD(45, 40,45, 60,180)
		COLOUR(White)
		STAR(16, 20,12, 40,80)
	FLAG_END(NP)
	FLAG_BEGIN(NR)
		FLOOD(NavyBlue)
		STAR(12, 40,20, 60,170)
		HBLOCK(Yellow, 110, 130)
	FLAG_END(NR)
	FLAG_BEGIN(NU)
		FLOOD(Yellow)
		BLIT(0,0, 120,120)
		COLOUR(NavyBlue)
		CIRCLE(16, 60,60)
		COLOUR(Yellow)
		STAR5(18, 60,60)
		STAR5(16, 60,21)
		STAR5(16, 30,61)
		STAR5(16, 90,61)
		STAR5(16, 60,101)
	FLAG_END(NU)
	FLAG_BEGIN(NZ)
		FLOOD(NavyBlue)
		BLIT(0,0, 120,120)
		STAR5(21, 180,38)
		STAR5(15, 208,88)
		STAR5(21, 150,103)
		STAR5(21, 180,200)
		COLOUR(Red)
		STAR5(15, 180,38)
		STAR5(10, 208,88)
		STAR5(15, 150,103)
		STAR5(15, 180,200)
	FLAG_END(NZ)
	FLAG_BEGIN(OM)
		HTRICOL(White, Red, Green)
		VBLOCK(Red, 0, 50)
		MIRROR(12), 25,15, 22,23, 22,42, 11,20, 8,23, 20,47, 10,50, 10,60, 20,63, 8,87, 11,90, 25,62,
	FLAG_END(OM)
	FLAG_BEGIN(PA)
		FLOOD(White)
		COLOUR(PureBlue)
		STAR5(36, 60,60)
		RECTANGLE(0,120, 120,240)
		COLOUR(Red)
		RECTANGLE(120,0, 240,120)
		STAR5(36, 180,180)
	FLAG_END(PA)
	FLAG_BEGIN(PE)
		VTRICOL(Red, White, Red)
	FLAG_END(PE)
	FLAG_BEGIN(PF)
		HQUADCOL(Red, White, White, Red)
		COLOUR(Yellow)
		STAR(19, 30,55, 120,120)
		COLOUR(PaleBlue)
		CHORD(55, 120,120, 70,170)
	FLAG_END(PF)	//Emblem
	FLAG_BEGIN(PG)
		FLOOD(Red)
		COLOUR(Black)
		TRIANGLE(0,0, 240,240)
		COLOUR(White)
		STAR5(15, 60,95)
		STAR5(15, 78,129)
		STAR5(15, 40,139)
		STAR5(15, 60,203)
		STAR5(10, 72,155)
	FLAG_END(PG) //Emblem
	FLAG_BEGIN(PH)
		HBICOL(PureBlue, Red)
		POLYGON(3), 0,0, 103,120, 0,240,
		COLOUR(Yellow)
		STAR(8, 40,20, 35,120)
		STAR5R(20, 10,40, 36)
		STAR5R(20, 80,120, 18)
		STAR5(20, 10,200)
	FLAG_END(PH)
	FLAG_BEGIN(PK)
		VTRICOL(White, Green, Green)
		CIRCLE(90, 160,120)
		COLOUR(Green)
		CIRCLE(80, 170,106)
		COLOUR(White)
		STAR5R(30, 195,75, 45)
	FLAG_END(PK)
	FLAG_BEGIN(PL)
		HBICOL(White, Red)
	FLAG_END(PL)
	FLAG_BEGIN(PR)
		HSTRIPE(5, Red, White)
		COLOUR(Blue)
		POLYGON(3), 0,0, 120,120, 0,240,
		COLOUR(White)
		STAR5R(45, 40,120, 0)
	FLAG_END(PR)
	FLAG_BEGIN(PT)
		FLOOD(Red)
		VBLOCK(Green, 0,90)
		COLOUR(Yellow)
		CIRCLE(70, 90,120)
		COLOUR(Green)
		CHORD(50, 90,120, 120,240)
		COLOUR(Red)
		CHORD(50, 90,120, 0,120)
		COLOUR(Red)
		SHIELD(70,80, 110,175, 0)
		COLOUR(White)
		SHIELD(80,100, 100,150, 0)
	FLAG_END(PT)
	FLAG_BEGIN(PW)
		FLOOD(Cyan)
		COLOUR(Yellow)
		CIRCLE(75, 105,120)
	FLAG_END(PW)
	FLAG_BEGIN(PY)
		HTRICOL(Red, White, Blue)
		COLOUR(Red)
		CIRCLE(35, 120,120)
		COLOUR(White)
		CIRCLE(25, 120,120)
		COLOUR(Blue)
		CIRCLE(15, 120,120)
		COLOUR(Yellow)
		STAR5(13, 120,120)
	FLAG_END(PY)
	FLAG_BEGIN(QA)
		FLOOD(Plum)
		POLYGON(12), 0,0, 75,0, 95,13, 75,27, 95,40, 75,53, 95,67, 75,80, 95,93, 75,107, 95,120, 0,120,
		POLYGON(12), 0,120, 95,120, 75,133, 95,147, 75,160, 95,173, 75,187, 95,200, 75,213, 95,227, 75,240, 0,240,
	FLAG_END(QA)
	FLAG_BEGIN(RE)
		VTRICOL(Blue, White, Red)
	FLAG_END(RE)
	FLAG_BEGIN(RO)
		VTRICOL(PureBlue, Yellow, Red)
	FLAG_END(RO)
	FLAG_BEGIN(RU)
		HTRICOL(White, PureBlue, Red)
	FLAG_END(RU)
	FLAG_BEGIN(RW)
		HQUADCOL(PaleBlue, PaleBlue, Yellow, Green)
		COLOUR(Yellow)
		STAR(14, 14,40, 200,60)
		STAR(14, 40,14, 200,60)
		COLOUR(PaleBlue)
		CIRCLE(16, 200,60)
		COLOUR(Yellow)
		CIRCLE(12, 200,60)
	FLAG_END(RW)
	FLAG_BEGIN(SA)
		FLOOD(Green)
		POLYGON(8), 62,160, 80,164, 180,164, 182,168, 182,184, 178,184, 178,170, 80,170,
		CIRCLE(6, 162,158)
		CIRCLE(6, 162,176)
	FLAG_END(SA)	//Emblem
	FLAG_BEGIN(SB)
		FLOOD(Yellow)
		COLOUR(PureBlue)
		TRIANGLE(220,0, 0,230)
		COLOUR(Green)
		TRIANGLE(20,240, 240,10)
		COLOUR(White)
		STAR5(20, 15,32)
		STAR5(20, 65,32)
		STAR5(20, 40,64)
		STAR5(20, 15,96)
		STAR5(20, 65,96)
	FLAG_END(SB)
	FLAG_BEGIN(SC)
		FLOOD(Red)
		TRIANGLE(0,240, 240,80)
		COLOUR(Green)
		TRIANGLE(0,240, 240,160)
		COLOUR(Yellow)
		TRIANGLE(160,0, 0,240)
		COLOUR(Blue)
		TRIANGLE(80,0, 0,240)
	FLAG_END(SC)
	FLAG_BEGIN(SD)
		HTRICOL(Red, White, Black)
		COLOUR(Green)
		POLYGON(3), 0,0, 80,120, 0,240,
	FLAG_END(SD)
	FLAG_BEGIN(SE)
		FLOOD(PaleBlue)
		COLOUR(Yellow)
		CROSS(12, 90)
	FLAG_END(SE)
	FLAG_BEGIN(SG)
		HBICOL(Red, White)
		CIRCLE(50, 50,60)
		COLOUR(Red)
		CIRCLE(50, 60,60)
		COLOUR(White)
		STAR5(12, 75,25)
		STAR5(12, 60,47)
		STAR5(12, 90,47)
		STAR5(12, 66,83)
		STAR5(12, 84,83)
	FLAG_END(SG)
	FLAG_BEGIN(SI)
		HTRICOL(White, Blue, Red)
		COLOUR(Blue)
		RECTANGLE(45,50, 75,120)
		COLOUR(White)
		MIRROR(6), 60,70, 56,80, 52,75, 45,95, 45,120, 60,120,
		COLOUR(Blue)
		SHIELD(45,80, 75,120, 1)
		COLOUR(Yellow)
		STAR5(6, 53,57)
		STAR5(6, 60,62)
		STAR5(6, 67,57)
	FLAG_END(SI)
	FLAG_BEGIN(SK)
		HTRICOL(White, Blue, Red)
		SHIELD(38,80, 102,185, 0)
		COLOUR(Red)
		RECTANGLE(40,55, 100,80)
		SHIELD(40,80, 100,180, 0)
		COLOUR(White)
		MIRROR(14), 70,66, 67,65, 68,86, 57,85, 58,90, 57,95, 68,94, 68,106, 54,105, 55,110, 54,115, 68,114, 67,161, 70,160,
	FLAG_END(SK)
	FLAG_BEGIN(SL)
		HTRICOL(Green, White, Blue)
	FLAG_END(SL)
	FLAG_BEGIN(SM)
		HBICOL(White, PaleBlue)	//Emblem
	FLAG_END(SM)
	FLAG_BEGIN(SN)
		VTRICOL(Green, Yellow, Red)
		COLOUR(Green)
		STAR5R(55, 120,120, 0)
	FLAG_END(SN)
	FLAG_BEGIN(SO)
		FLOOD(PaleBlue)
		STAR5R(55, 120,120, 0)
	FLAG_END(SO)
	FLAG_BEGIN(SR)
		HTRICOL(Green, Red, Green)
		HBLOCK(White, 74,86)
		HBLOCK(White, 154,166)
		COLOUR(Yellow)
		STAR5(36, 120,123)
	FLAG_END(SR)
	FLAG_BEGIN(ST)
		HTRICOL(Green, Yellow, Green)
		COLOUR(Red)
		POLYGON(3), 0,0, 60,120, 0,240,
		COLOUR(Black)
		STAR5(36, 120,123)
		STAR5(36, 180,123)
	FLAG_END(ST)
	FLAG_BEGIN(SV)
		HTRICOL(PureBlue, White, PureBlue)	//Emblem
	FLAG_END(SV)
	FLAG_BEGIN(SY)
		HTRICOL(Red, White, Black)
		COLOUR(Green)
		STAR5(36, 80,123)
		STAR5(36, 160,123)
	FLAG_END(SY)
	FLAG_BEGIN(SZ)
		HQUADCOL(PaleBlue, Plum, Plum, PaleBlue)
		HBLOCK(Yellow, 45,60)
		HBLOCK(Yellow, 180,195)
		POLYGON(14), 120,70, 70,120, 120,170, 130,168, 140,165, 150,159, 160,150, 170,136, 180,120, 170,104, 160,90, 150,81, 140,75, 130,72,
		COLOUR(Black)
		POLYGON(17), 120,70, 118,80, 127,120, 122,159, 130,168, 120,170, 110,168, 100,165, 90,159, 80,150, 70,136, 60,120, 70,104, 80,90, 90,81, 100,75, 110,72,
		RECTANGLE(130,100, 158,140)
		COLOUR(White)
		RECTANGLE(130,115, 158,125)
		RECTANGLE(133,100, 135,140)
		RECTANGLE(138,100, 140,140)
		RECTANGLE(143,100, 145,140)
		RECTANGLE(148,100, 150,140)
		RECTANGLE(153,100, 155,140)
		RECTANGLE(82,100, 110,140)
		COLOUR(Black)
		RECTANGLE(82,115, 110,125)
		RECTANGLE(85,100, 87,140)
		RECTANGLE(90,100, 92,140)
		RECTANGLE(95,100, 97,140)
		RECTANGLE(100,100, 102,140)
		RECTANGLE(105,100, 107,140)
	FLAG_END(SZ)	//Emblem
	FLAG_BEGIN(TC)
		FLOOD(NavyBlue)
		BLIT(0,0, 120,120)	//Emblem
		COLOUR(Yellow)
		RECTANGLE(160,70, 200,120)
		SHIELD(160,120, 200,170, 0)
	FLAG_END(TC)
	FLAG_BEGIN(TD)
		VTRICOL(Blue, Yellow, Red)
	FLAG_END(TD)
	FLAG_BEGIN(TG)
		HSTRIPE(5, Green, Yellow)
		COLOUR(Red)
		RECTANGLE(0,0, 72,144)
		COLOUR(White)
		STAR5(36, 36,72)
	FLAG_END(TG)
	FLAG_BEGIN(TH)
		HTRICOL(Red, Blue, Red)
		HBLOCK(White, 40, 80)
		HBLOCK(White, 160, 200)
	FLAG_END(TH)
	FLAG_BEGIN(TJ)
		HQUADCOL(Red, White, White, Green)	//Emblem
	FLAG_END(TJ)
	FLAG_BEGIN(TM)
		FLOOD(Green)
		VBLOCK(Plum, 22, 66)
		CIRCLE(35, 100,55)
		COLOUR(Green)
		CIRCLE(35, 95,50)
		COLOUR(White)
		STAR(5, 2,6, 93,30)
		STAR(5, 2,6, 100,39)
		STAR(5, 2,6, 93,45)
		STAR(5, 2,6, 86,51)
		STAR(5, 2,6, 93,60)
	FLAG_END(TM)	//Emblem
	FLAG_BEGIN(TN)
		FLOOD(Red)
		CIRCLE(75, 120,120)
		COLOUR(Red)
		CIRCLE(55, 120,120)
		COLOUR(White)
		CIRCLE(45, 128,120)
		COLOUR(Red)
		STAR5R(35, 130,120, 54)
	FLAG_END(TN)
	FLAG_BEGIN(TO)
		FLOOD(Red)
		RECTANGLE(0,0, 100,120)
		COLOUR(Red)
		RECTANGLE(25,40, 75,80)
		RECTANGLE(40,10, 60,110)
	FLAG_END(TO)
	FLAG_BEGIN(TR)
		FLOOD(Red)
		CIRCLE(60, 75,120)
		COLOUR(Red)
		CIRCLE(50, 85,120)
		COLOUR(White)
		STAR5R(30, 120,120, 54)
	FLAG_END(TR)
	FLAG_BEGIN(TT)
		FLOOD(Red)
		POLYGON(4), 0,0, 60,0, 240,240, 180,240,
		COLOUR(Black)
		POLYGON(4), 15,0, 45,0, 225,240, 195,240,
	FLAG_END(TT)
	FLAG_BEGIN(TV)
		FLOOD(Cyan)
		BLIT(0,0, 120,120)	//Emblem
		COLOUR(Yellow)
		STAR5(20, 224,31)
		STAR5R(20, 202,67, 36)
		STAR5R(20, 181,84, 36)
		STAR5(20, 224,126)
		STAR5R(20, 155,153, 36)
		STAR5(20, 202,153)
		STAR5R(20, 181,191, 36)
		STAR5R(20, 155,203, 36)
		STAR5(20, 131,215)
	FLAG_END(TV)
	FLAG_BEGIN(TW)
		FLOOD(Red)
		COLOUR(NavyBlue)
		RECTANGLE(0,0, 120,120)
		COLOUR(White)
		STAR(12, 55,30, 60,60)
		COLOUR(NavyBlue)
		CIRCLE(39, 60,60)
		COLOUR(White)
		CIRCLE(35, 60,60)
	FLAG_END(TW)
	FLAG_BEGIN(TZ)
		FLOOD(PaleBlue)
		COLOUR(Green)
		TRIANGLE(240,0, 0,240)
		COLOUR(Yellow)
		DIAGONAL(2, 83)
		COLOUR(Black)
		DIAGONAL(2, 60)
	FLAG_END(TZ)
	FLAG_BEGIN(UA)
		HBICOL(PaleBlue, Yellow)
	FLAG_END(UA)
	FLAG_BEGIN(UG)
		FLOOD(Black)
		HBLOCK(Yellow, 40, 80)
		HBLOCK(Red, 80, 120)
		HBLOCK(Yellow, 160, 200)
		HBLOCK(Red, 200, 240)
		CIRCLE(40, 120,120)
	FLAG_END(UG)
	FLAG_BEGIN(US)
		HSTRIPE(13, Red, White)
		SUBFLAG(US_Stars, 1.488372)
		BLIT(0,0, 96,129)
	FLAG_END(US)
	FLAG_BEGIN(UY)
		HSTRIPE(9, White, PureBlue)
		RECTANGLE(0,0, 89,133)
		COLOUR(Yellow)
		STAR(16, 55,25, 44,66)
	FLAG_END(UY)
	FLAG_BEGIN(UZ)
		HTRICOL(Blue, White, Green)
		CIRCLE(30, 25,39)
		COLOUR(Blue)
		CIRCLE(25, 30,39)
		COLOUR(White)
		STARLINE(10, 66,18, 3, 11)
		STARLINE(10, 55,39, 4, 11)
		STARLINE(10, 44,60, 5, 11)
		HBLOCK(Plum, 76,84)
		HBLOCK(Plum, 156,164)
	FLAG_END(UZ)
	FLAG_BEGIN(VA)
		VBICOL(Yellow, White)	//Emblem
	FLAG_END(VA)
	FLAG_BEGIN(VC)
		VQUADCOL(PureBlue, Yellow, Yellow, Green)
		COLOUR(Green)
		POLYGON(4), 96,48, 116,96, 96,144, 76,96,
		POLYGON(4), 144,48, 164,96, 144,144, 124,96,
		POLYGON(4), 120,104, 140,152, 120,200, 100,152,
	FLAG_END(VC)
	FLAG_BEGIN(VE)
		HTRICOL(Yellow, PureBlue, Red)
		STAR5(15, 58,146)
		STAR5(15, 76,119)
		STAR5(15, 96,100)
		STAR5(15, 120,94)
		STAR5(15, 144,100)
		STAR5(15, 164,119)
		STAR5(15, 182,146)
	FLAG_END(VE)
	FLAG_BEGIN(VG)
		FLOOD(NavyBlue)
		BLIT(0,0, 120,120)	//Emblem
		COLOUR(Green)
		RECTANGLE(160,70, 200,120)
		SHIELD(160,120, 200,170, 0)
	FLAG_END(VG)
	FLAG_BEGIN(VI)
		FLOOD(White)
		COLOUR(Yellow)
		MIRROR(10), 120,90, 90,66, 15,45, 50,95, 95,127, 67,173, 83,176, 109,143, 96,172, 120,200,
		POLYGON(6), 105,90, 112,53, 102,54, 113,38, 122,38, 135,90,
		COLOUR(Blue)
		RECTANGLE(92,76, 148,105)
		SHIELD(92,105, 148,162, 0)
		COLOUR(White)
		SHIELD(93,105, 147,160, 0)
	FLAG_END(VI)
	FLAG_BEGIN(VN)
		FLOOD(Red)
		COLOUR(Yellow)
		STAR5R(80, 120,120, 0)
	FLAG_END(VN)
	FLAG_BEGIN(VU)
		FLOOD(Black)
		COLOUR(Red)
		POLYGON(4), 5,0, 240,0, 240,105, 110,105,
		COLOUR(Green)
		POLYGON(4), 5,240, 240,240, 240,135, 110,135,
		COLOUR(Yellow)
		POLYGON(9), 0,5, 108,113, 240,113, 240,127, 108,127, 0,235, 0,222, 102,120, 0,18,
		CIRCLE(40, 36,120)
		COLOUR(Black)
		CIRCLE(32, 36,120)
		COLOUR(Yellow)
		STAR(4, 8,20, 36,120)
	FLAG_END(VU)
	FLAG_BEGIN(WS)
		FLOOD(Red)
		COLOUR(Blue)
		RECTANGLE(0,0, 120,120)
		COLOUR(White)
		STAR5(10, 60,16)
		STAR5(10, 75,45)
		STAR5(10, 45,52)
		STAR5(7, 66,70)
		STAR5(10, 60,102)
	FLAG_END(WS)
	FLAG_BEGIN(YE)
		HTRICOL(Red, White, Black)
	FLAG_END(YE)
	FLAG_BEGIN(ZA)
		FLOOD(Green)
		POLYGON(4), 30,0, 122,92, 240,92, 240,0,
		POLYGON(4), 30,240, 122,148, 240,148, 240,240,
		COLOUR(Red)
		POLYGON(4), 50,0, 125,75, 240,75, 240,0,
		COLOUR(PureBlue)
		POLYGON(4), 50,240, 125,165, 240,165, 240,240,
		COLOUR(Yellow)
		POLYGON(3), 0,35, 85,120, 0,205,
		COLOUR(Black)
		POLYGON(3), 0,55, 65,120, 0,185,
	FLAG_END(ZA)
	FLAG_BEGIN(ZM)
		FLOOD(Green)
		COLOUR(Red)
		RECTANGLE(159,80, 240,240)
		COLOUR(Black)
		RECTANGLE(186,80, 240,240)
		COLOUR(Orange)
		RECTANGLE(213,80, 240,240)
		POLYGON(18), 196,30, 180,20, 165,28, 198,50, 197,60, 199,59, 199,50, 201,50, 201,59, 203,60, 202,50, 235,28, 220,20, 204,30, 203,25, 205,25, 202,20, 198,20,
	FLAG_END(ZM)
	FLAG_BEGIN(ZW)
		FLOOD(Green)
		HBLOCK(Yellow, 35,205)
		HBLOCK(Red, 69,171)
		COLOUR(Black)
		POLYGON(6), 0,0, 68,102, 240,102, 240,138, 68,138, 0,240,
		COLOUR(White)
		POLYGON(3), 0,6, 76,120, 0,234,
		COLOUR(Red)
		STAR5R(50, 27,120, 0)
	FLAG_END(ZW)
	FLAG_BEGIN(US_Stars)
		FLOOD(Blue)
		STARLINE(17, 20,27, 6, 40)
		STARLINE(17, 40,50, 5, 40)
		STARLINE(17, 20,73, 6, 40)
		STARLINE(17, 40,96, 5, 40)
		STARLINE(17, 20,119, 6, 40)
		STARLINE(17, 40,142, 5, 40)
		STARLINE(17, 20,165, 6, 40)
		STARLINE(17, 40,188, 5, 40)
		STARLINE(17, 20,211, 6, 40)
	FLAG_END(US_Stars)
	0xFF
};
local const BYTE* raw_flag[Flag_Count];

typedef struct _MAPFLAG
{
	HDC hdc;
	const BYTE* p;
	HBRUSH* hbrush;
	HPEN* hpen;
	int brush;
	int brush_selected;
	double xscale;
	double yscale;
	double aspect;
	int xpos;
	int ypos;
	struct _MAPFLAG* subflag;
	HBITMAP subbmp;
}
MAPFLAG;

// Forward declarations
local void DrawFlag(MAPFLAG* flag);

local LONG DrawOrdinateX(MAPFLAG* flag, double x)
{
	// Fetch an x ordinate
	ASSERT((x >= 0) && (x <= 240));
	return (LONG)(x * flag->xscale + 0.5);
}

local LONG DrawOrdinateY(MAPFLAG* flag, double y)
{
	// Fetch a y ordinate
//	ASSERT((y >= 0) && (y <= 240));
	return (LONG)(y * flag->yscale + 0.5);
}

local LONG DrawNextOrdinateX(MAPFLAG* flag)
{
	// Fetch the next x ordinate
	return DrawOrdinateX(flag, *(flag->p++));
}

local LONG DrawNextOrdinateY(MAPFLAG* flag)
{
	// Fetch the next y ordinate
	return DrawOrdinateY(flag, *(flag->p++));
}

local HBRUSH DrawGetBrush(MAPFLAG* flag, enum FlagColour colour)
{
	static const COLORREF rgb[16] =
	{
		PALETTERGB(0x00, 0x00, 0x00),	// Black
		PALETTERGB(0x00, 0x00, 0xFF),	// PureBlue
		PALETTERGB(0xFF, 0x00, 0x00),	// Red
		PALETTERGB(0xCC, 0x00, 0x33),	// Magenta
		PALETTERGB(0x00, 0x99, 0x00),	// Green
		PALETTERGB(0x66, 0xCC, 0xFF),	// Cyan
		PALETTERGB(0xFF, 0xFF, 0x00),	// Yellow
		PALETTERGB(0xFF, 0xFF, 0xFF),	// White
		PALETTERGB(0x00, 0x00, 0x66),	// NavyBlue
		PALETTERGB(0xCC, 0x33, 0x33),	// Plum
		PALETTERGB(0xFF, 0x99, 0x00),	// Orange
		PALETTERGB(0x00, 0x99, 0xFF),	// PaleBlue
		PALETTERGB(0x00, 0x00, 0x99),	// Blue
		PALETTERGB(0xAA, 0x22, 0x33),	// Brown
		PALETTERGB(0xCC, 0xCC, 0xCC),	// Grey
		PALETTERGB(0x00, 0x00, 0x00)	// Unused
	};

	// If necessary create the brush and pen
	ASSERT((colour >= 0) && (colour <= 15));
	if (flag->hbrush[colour] == NULL)
	{
		// Create the brush and pen in tandem
		ASSERT(flag->hpen[colour] == NULL);
		flag->hbrush[colour] = OS(CreateSolidBrush)(rgb[colour]);
		ASSERT(flag->hbrush[colour] != NULL);
		flag->hpen[colour] = OS(CreatePen)(PS_SOLID, 0, rgb[colour]);
		ASSERT(flag->hpen[colour] != NULL);
	}
	return flag->hbrush[colour];
}

local void DrawSelectBrush(MAPFLAG* flag)
{
	// Set the current brush
	if (flag->hdc != NULL)
	{
		ASSERT((flag->brush >= 0) && (flag->brush <= 15));
		if (flag->brush != flag->brush_selected)
		{
			OS(SelectObject)(flag->hdc, DrawGetBrush(flag, flag->brush));
			OS(SelectObject)(flag->hdc, flag->hpen[flag->brush]);
			flag->brush_selected = flag->brush;
		}
	}
	return;
}

local void DrawRectangle(MAPFLAG* flag, int x0, int y0, int x1, int y1, enum FlagColour colour)
{
	// Set the current brush
	if (flag->hdc != NULL)
	{
		RECT rect;
		rect.left = DrawOrdinateX(flag, x0);
		rect.top = DrawOrdinateY(flag, y0);
		rect.right = DrawOrdinateX(flag, x1);
		rect.bottom = DrawOrdinateY(flag, y1);
		VERIFY(OS(FillRect)(flag->hdc, &rect, DrawGetBrush(flag, colour)));
	}
	return;
}

local void DrawStarEx(MAPFLAG* flag, int n, double x, double y, double r1, double r2, double rot)
{
	// Draw a five-pointed star
	if (flag->hdc != NULL)
	{
		POINT pt[40];
		int i;
		ASSERT((n * 2) <= dimof(pt));
		for (i = 0; i < (n * 2); i++)
		{
			double angle = M_PI * i / n + rot;
			double ry = r1;
			double rx = ry / flag->aspect;
			r1 = r2;
			r2 = ry;
			pt[i].x = DrawOrdinateX(flag, x + sin(angle) * rx);
			pt[i].y = DrawOrdinateY(flag, y - cos(angle) * ry);
		}
		VERIFY(OS(Polygon)(flag->hdc, pt, i));
	}
	return;
}

local void DrawStar(MAPFLAG* flag, int n, double r1, double r2)
{
	// Draw a simple five-pointed star at the current position
	if (flag->hdc != NULL)
	{
		DrawSelectBrush(flag);
		DrawStarEx(flag, n, flag->xpos, flag->ypos, r1, r2, 0.0);
	}
	return;
}

local void DrawShield(MAPFLAG* flag, double x0, double y0, double x1, double y1, BOOL outside)
{
	// Draw the lower part of a shield
	if (flag->hdc != NULL)
	{
		POINT pt[22];
		int i;
		for (i = 0; i <= 20; i++)
		{
			double h = sqrt(i * (80 - i)) * 0.02886751346;
			pt[i].x = DrawOrdinateX(flag, x0 + i * 0.05 * (x1 - x0));
			pt[i].y = DrawOrdinateY(flag, y0 + h * (y1 - y0));
		}
		pt[21].x = DrawOrdinateX(flag, outside ? x0 : x1);
		pt[21].y = DrawOrdinateY(flag, outside ? y1 : y0);
		VERIFY(OS(Polygon)(flag->hdc, pt, 22));
	}
	return;
}

local void DrawCreateSubflag(MAPFLAG* flag, enum Flag content, int width, int height)
{
	// Create a new subflag
	HBITMAP hbitmap = OS(CreateCompatibleBitmap)(flag->hdc, width, height);
	ASSERT(flag != NULL);
	ASSERT(flag->subflag != NULL);
	ASSERT(hbitmap != NULL);
	flag->subflag->hdc = OS(CreateCompatibleDC)(flag->hdc);
	flag->subflag->p = raw_flag[content];
	flag->subflag->hbrush = flag->hbrush;
	flag->subflag->hpen = flag->hpen;
	flag->subflag->xscale = width / 240.0;
	flag->subflag->yscale = height / 240.0;
	flag->subflag->aspect = (double)width / height;
	flag->subflag->xpos = 0;
	flag->subflag->ypos = 0;
	flag->subflag->subflag = NULL;
	flag->subbmp = OS(SelectObject)(flag->subflag->hdc, hbitmap);
	ASSERT(flag->subbmp != NULL);
	DrawFlag(flag->subflag);
	flag->subflag->xpos = width;
	flag->subflag->ypos = height;
	VERIFY(OS(SetStretchBltMode)(flag->subflag->hdc, STRETCH_HALFTONE));
	return;
}

local void DrawDeleteSubflag(MAPFLAG* flag)
{
	// Delete resources used by any subflag
	if (flag->subflag->hdc != NULL)
	{
		// Clean up the subflag
		HBITMAP hbitmap = OS(SelectObject)(flag->subflag->hdc, flag->subbmp);
		VERIFY(OS(DeleteObject)(hbitmap));
		VERIFY(OS(DeleteDC)(flag->subflag->hdc));
		ASSERT(flag->subflag->subflag == NULL);
		flag->subflag->hdc = NULL;
	}
	return;
}

local void DrawFlagFirst(MAPFLAG* flag)
{
	// First instruction for a flag
	int hi = HI(*(flag->p));
	int lo = LO(*(flag->p++));
	switch (hi)
	{
		case FlagFirstExtend:
			switch (lo)
			{
				case FlagFirstExtendNull:
					// Do nothing (non-rectangular flag)
					break;
				case FlagFirstExtendH2:
					// Horizontal bicolor
					DrawRectangle(flag, 0, 0, 240, 120, HI(*(flag->p)));
					DrawRectangle(flag, 0, 120, 240, 240, LO(*(flag->p++)));
					break;
				case FlagFirstExtendV2:
					// Vertical bicolor
					DrawRectangle(flag, 0, 0, 120, 240, HI(*(flag->p)));
					DrawRectangle(flag, 120, 0, 240, 240, LO(*(flag->p++)));
					break;
				case FlagFirstExtendH4:
					// Horizontal quadcolor
					DrawRectangle(flag, 0, 0, 240, 60, HI(*(flag->p)));
					DrawRectangle(flag, 0, 60, 240, 120, LO(*(flag->p++)));
					DrawRectangle(flag, 0, 120, 240, 180, HI(*(flag->p)));
					DrawRectangle(flag, 0, 180, 240, 240, LO(*(flag->p++)));
					break;
				case FlagFirstExtendV4:
					// Vertital quadcolor
					DrawRectangle(flag, 0, 0, 60, 240, HI(*(flag->p)));
					DrawRectangle(flag, 60, 0, 120, 240, LO(*(flag->p++)));
					DrawRectangle(flag, 120, 0, 180, 240, HI(*(flag->p)));
					DrawRectangle(flag, 180, 0, 240, 240, LO(*(flag->p++)));
					break;
				case FlagFirstExtendHStripe:
					// Horizontal stripes
					{
						double dy = 240.0 / *(flag->p++);
						int c1 = HI(*(flag->p));
						int c2 = LO(*(flag->p++));
						double y = 0.5;
						do
						{
							DrawRectangle(flag, 0, (int)y, 240, (int)(y + dy), c1);
							y += dy;
							if (y < 240)
							{
								DrawRectangle(flag, 0, (int)y, 240, (int)(y + dy), c2);
								y += dy;
							}
						}
						while (y < 240);
					}
					break;
				default:
					UNREACHABLE();
			}
			break;
		case FlagFirstFlood:
			// Fill the whole flag
			DrawRectangle(flag, 0, 0, 240, 240, lo);
			break;
		case FlagFirstH3:
			// Horizontal tricolor
			DrawRectangle(flag, 0, 0, 240, 80, lo);
			DrawRectangle(flag, 0, 80, 240, 160, HI(*(flag->p)));
			DrawRectangle(flag, 0, 160, 240, 240, LO(*(flag->p++)));
			break;
		case FlagFirstV3:
			// Vertical tricolor
			DrawRectangle(flag, 0, 0, 80, 240, lo);
			DrawRectangle(flag, 80, 0, 160, 240, HI(*(flag->p)));
			DrawRectangle(flag, 160, 0, 240, 240, LO(*(flag->p++)));
			break;
		default:
			UNREACHABLE();
	}
	return;
}

local void DrawFlagNext(MAPFLAG* flag)
{
	// Subsequent instructions for a flag
	int hi = HI(*(flag->p));
	int lo = LO(*(flag->p++));
	switch (hi)
	{
		case FlagExtend:
			switch (lo)
			{
				case FlagExtendTriangle:
					// Triangle
					{
						POINT pt[3];
						pt[0].x = DrawNextOrdinateX(flag);
						pt[0].y = DrawNextOrdinateY(flag);
						pt[1].x = DrawNextOrdinateX(flag);
						pt[1].y = DrawNextOrdinateY(flag);
						if (flag->hdc != NULL)
						{
							int i = (((pt[1].x - pt[0].x) * (pt[1].y - pt[0].y)) < 0);
							pt[2].x = pt[i].x;
							pt[2].y = pt[1-i].y;
							DrawSelectBrush(flag);
							VERIFY(OS(Polygon)(flag->hdc, pt, dimof(pt)));
						}
					}
					break;
				case FlagExtendRectangle:
					// Rectangle
					DrawRectangle(flag, flag->p[0], flag->p[1], flag->p[2], flag->p[3], flag->brush);
					flag->p += 4;
					break;
				case FlagExtendDiagonal:
					// Triangle
					{
						POINT pt[6];
						int i = *(flag->p++);
						int w = (i & 0x3F) + 20;
						pt[0].x = DrawOrdinateX(flag, 0);
						pt[0].y = DrawOrdinateY(flag, 0);
						pt[1].x = DrawOrdinateX(flag, w);
						pt[1].y = DrawOrdinateY(flag, 0);
						pt[2].x = DrawOrdinateX(flag, 240);
						pt[2].y = DrawOrdinateY(flag, 240 - w);
						pt[3].x = DrawOrdinateX(flag, 240);
						pt[3].y = DrawOrdinateY(flag, 240);
						pt[4].x = DrawOrdinateX(flag, 240 - w);
						pt[4].y = DrawOrdinateY(flag, 240);
						pt[5].x = DrawOrdinateX(flag, 0);
						pt[5].y = DrawOrdinateY(flag, w);
						if (flag->hdc != NULL)
						{
							DrawSelectBrush(flag);
							if (i & 0x40)
								VERIFY(OS(Polygon)(flag->hdc, pt, dimof(pt)));
							if (i & 0x80)
							{
								for (i = 0; i < dimof(pt); i++)
									pt[i].x = (LONG)(flag->xscale * 240 - pt[i].x);
								VERIFY(OS(Polygon)(flag->hdc, pt, dimof(pt)));
							}
						}
					}
					break;
				case FlagExtendBlit:
					// Blit
					{
						RECT rect;
						if ((flag->subflag != NULL) && (flag->subflag->hdc == NULL))
						{
							// No subflag created, so use the union flag
							DrawCreateSubflag(flag, Flag_GB, 1024, 512);
						}
						rect.left = DrawOrdinateX(flag, *(flag->p++));
						rect.top = DrawOrdinateY(flag, *(flag->p++));
						rect.right = DrawOrdinateX(flag, *(flag->p++));
						rect.bottom = DrawOrdinateY(flag, *(flag->p++));
						if (flag->hdc != NULL)
						{
							ASSERT(flag->subflag != NULL);
							VERIFY(OS(StretchBlt)(flag->hdc, rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top,
												  flag->subflag->hdc, 0, 0, flag->subflag->xpos, flag->subflag->ypos, SRCCOPY));
						}
					}
					break;
				case FlagExtendStar5r:
					// Larger, rotated five pointed star
					{
						double r1 = *(flag->p++);
						double r2 = r1 * 0.385;
						double x = *(flag->p++);
						double y = *(flag->p++);
						double rot = *(flag->p++) * M_PI / 180;
						DrawSelectBrush(flag);
						DrawStarEx(flag, 5, x, y, r1, r2, rot);
					}
					break;
				case FlagExtendStar5x:
					// Circle of five pointed stars
					{
						int rot = *(flag->p) & 0x80;
						int i, n = *(flag->p++) & 0x7F;
						double x = *(flag->p++);
						double y = *(flag->p++);
						double r0 = *(flag->p++);
						double r1 = *(flag->p++);
						double r2 = r1 * 0.385;
						DrawSelectBrush(flag);
						for (i = 0; i < n; i++)
						{
							double angle = M_PI * 2 * i / n;
							DrawStarEx(flag, 5, x + sin(angle) * r0 / flag->aspect, y - cos(angle) * r0, r1, r2, rot ? angle : 0.0);
						}
					}
					break;
				case FlagExtendStarX:
					// Circle of stars
					{
						int i, n = *(flag->p++);
						double x = *(flag->p++);
						double y = *(flag->p++);
						double r0 = *(flag->p++);
						double r1 = *(flag->p++);
						double r2 = *(flag->p++);
						DrawSelectBrush(flag);
						for (i = 0; i < n; i++)
						{
							double angle = M_PI * 2 * i / n;
							DrawStarEx(flag, 5, x + sin(angle) * r0 / flag->aspect, y - cos(angle) * r0, r1, r2, angle);
						}
					}
					break;
				case FlagExtendChord:
					// Chord of a circle
					{
						double ry = *(flag->p++);
						double rx = ry / flag->aspect;
						double x = *(flag->p++);
						double y = *(flag->p++);
						double a1 = *(flag->p++) * M_PI / 120;
						double a2 = *(flag->p++) * M_PI / 120;
						RECT rect;
						POINT pt1, pt2;
						rect.left = DrawOrdinateX(flag, x - rx);
						rect.top = DrawOrdinateY(flag, y - ry);
						rect.right = DrawOrdinateX(flag, x + rx);
						rect.bottom = DrawOrdinateY(flag, y + ry);
						pt1.x = DrawOrdinateX(flag, x + sin(a1) * rx);
						pt1.y = DrawOrdinateY(flag, y - cos(a1) * ry);
						pt2.x = DrawOrdinateX(flag, x + sin(a2) * rx);
						pt2.y = DrawOrdinateY(flag, y - cos(a2) * ry);
						if (flag->hdc != NULL)
						{
							DrawSelectBrush(flag);
							VERIFY(OS(Chord)(flag->hdc, rect.left, rect.top, rect.right, rect.bottom,
											 pt2.x, pt2.y, pt1.x, pt1.y));
						}
					}
					break;
				case FlagExtendSubflag:
					// Subflag
					{
						int id = *(flag->p++);
						int sx = 1024;
						int sy = (sx * 240) / (*(flag->p++) + 240);
						if (flag->hdc != NULL)
						{
							DrawCreateSubflag(flag, id, sx, sy);
						}
					}
					break;
				default:
					UNREACHABLE();
			}
			break;
		case FlagSetBrush:
			// Select a different colour
			flag->brush = lo;
			break;
		case FlagCross:
			// Vertical cross
			{
				int w = (lo + 1) * 2;
				int c = *(flag->p++);
				DrawRectangle(flag, c - w, 0, c + w, 240, flag->brush);
				w *= 2;
				DrawRectangle(flag, 0, 120 - w, 240, 120 + w, flag->brush);
			}
			break;
		case FlagPolygon:
			// Polygon
			{
				POINT pt[19];
				int i, n = lo + 3;
				for (i = 0; i < n; i++)
				{
					pt[i].x = DrawNextOrdinateX(flag);
					pt[i].y = DrawNextOrdinateY(flag);
				}
				if (flag->hdc != NULL)
				{
					DrawSelectBrush(flag);
					VERIFY(OS(Polygon)(flag->hdc, pt, n));
				}
			}
			break;
		case FlagMirror:
			// Mirrored polygon
			{
				POINT pt[36];
				int i, n = lo + 3;
				for (i = 0; i < n; i++)
				{
					pt[i].x = DrawNextOrdinateX(flag);
					pt[i].y = DrawNextOrdinateY(flag);
				}
				if (flag->hdc != NULL)
				{
					for (i = 0; i < (n - 2); i++)
					{
						pt[n + i].x = pt[0].x * 2 - pt[n - i - 2].x;
						pt[n + i].y = pt[n - i - 2].y;
					}
					DrawSelectBrush(flag);
					VERIFY(OS(Polygon)(flag->hdc, pt, n * 2 - 2));
				}
			}
			break;
		case FlagHBlock:
			// Horizontal block
			{
				int y0 = *(flag->p++);
				int y1 = *(flag->p++);
				DrawRectangle(flag, 0, y0, 240, y1, lo);
			}
			break;
		case FlagVBlock:
			// Vertical block
			{
				int x0 = *(flag->p++);
				int x1 = *(flag->p++);
				DrawRectangle(flag, x0, 0, x1, 240, lo);
			}
			break;
		case FlagStar:
			// Star
			{
				double r1 = *(flag->p++);
				double r2 = *(flag->p++);
				flag->xpos = *(flag->p++);
				flag->ypos = *(flag->p++);
				DrawStar(flag, lo + 4, r1, r2);
			}
			break;
		case FlagStar5:
			// Five pointed star
			{
				double r = (lo + 3) * 2.0;
				flag->xpos = *(flag->p++);
				flag->ypos = *(flag->p++);
				DrawStar(flag, 5, r, r * 0.385);
			}
			break;
		case FlagStarLine:
			// Line of stars
			{
				int d = *(flag->p++);
				double r = *(flag->p++);
				flag->xpos = *(flag->p++);
				flag->ypos = *(flag->p++);
				while (lo-- > 0)
				{
					DrawStar(flag, 5, r, r * 0.385);
					flag->xpos += d;
				}
			}
			break;
		case FlagShield:
			// Shield tip
			{
				double x0 = *(flag->p++);
				double y0 = *(flag->p++);
				double x1 = *(flag->p++);
				double y1 = *(flag->p++);
				double xm = (x0 + x1) / 2;
				DrawSelectBrush(flag);
				DrawShield(flag, x0, y0, xm, y1, lo);
				DrawShield(flag, x1, y0, xm, y1, lo);
			}
			break;
		default:
			UNREACHABLE();
	}
	return;
}

local void DrawFlag(MAPFLAG* flag)
{
	// Draw a flag into a new device context
	flag->brush = White;
	flag->brush_selected = 0xFF;
	DrawFlagFirst(flag);
	while (*(flag->p) < FlagFirstMarker)
	{
		// Execute each instruction
		DrawFlagNext(flag);
	}
	OS(SelectObject)(flag->hdc, OS(GetStockObject)(BLACK_PEN));
	OS(SelectObject)(flag->hdc, OS(GetStockObject)(WHITE_BRUSH));
	return;
}

void FlagDraw(HDC hdc, int width, int height, double aspect, int flag)
{
	HBRUSH hbrush[16];
	HPEN hpen[16];
	MAPFLAG fs, sfs;
	int i;

	// Fill in a structure and forward on
	fs.hdc = hdc;
	fs.p = raw_flag[flag];
	fs.hbrush = hbrush;
	fs.hpen = hpen;
	fs.xscale = width / 240.0;
	fs.yscale = height / 240.0;
	fs.aspect = aspect;
	fs.xpos = 0;
	fs.ypos = 0;
	fs.subflag = &sfs;
	sfs.hdc = NULL;
	for (i = 0; i < dimof(hbrush); i++)
	{
		// No brushes/pens initially used
		hbrush[i] = NULL;
		hpen[i] = NULL;
	}
	DrawFlag(&fs);
	DrawDeleteSubflag(&fs);
	for (i = 0; i < dimof(hbrush); i++)
	{
		// Clean up used brushes/pens
		if (hbrush[i] != NULL)
			VERIFY(OS(DeleteObject)(hbrush[i]));
		if (hpen[i] != NULL)
			VERIFY(OS(DeleteObject)(hpen[i]));
	}
	return;
}

void FlagDecompress(void)
{
	// Record pointers to the beginning of each flag
	MAPFLAG flag;
	int i;
	memset(&flag, 0, sizeof(flag));
	flag.p = raw_flag_data;
	flag.aspect = 2;
	for (i = 0; i < Flag_Count; i++)
	{
		raw_flag[i] = flag.p;
		DrawFlagFirst(&flag);
		while (*flag.p < FlagFirstMarker)
		{
			// Execute each instruction
			DrawFlagNext(&flag);
		}
	}
	return;
}

